#include <cstdio>
#include <iostream>
#include <fstream>
#include "gluvi.h"
#include "simulation.h"

using namespace std;

Simulation sim;
float dt;

struct MySlider: public Gluvi::Slider
{
    float maxvalue;
    float &control;

    MySlider(const char *text_, float maxvalue_, float &control_)
        : Gluvi::Slider(text_), maxvalue(maxvalue_), control(control_)
    {}

    virtual void action()
    {
        control=maxvalue*position/float(length);
    }
};

void init_simulation()
{
   dt=0.02;
   sim.init(200);
}

void advance_simulation(int ignored)
{
   sim.advance(dt);
   glutPostRedisplay();
   glutTimerFunc(50, advance_simulation, 0);
}

void display(void)
{
   glDisable(GL_LIGHTING);

   // draw the box the simulation is supposed to stay inside
   glColor3f(1, 1, 0.5);
   glBegin(GL_LINE_STRIP);
   glVertex2f(0, 0);
   glVertex2f(1, 0);
   glVertex2f(1, 1);
   glVertex2f(0, 1);
   glVertex2f(0, 0);
   glEnd();

   // draw the water surface
   glBegin(GL_LINE_STRIP);
   glColor3f(0.75, 0.75, 1);
   for(int i=0; i<sim.n; ++i)
      glVertex2f((i+0.5)*sim.dx+sim.disp(i), sim.h(i));
   glEnd();

   glPointSize(20);
   glBegin(GL_POINTS);
   glColor3f(1,0.5,0.5);
   int i=sim.n/3;
   glVertex2f((i+0.5)*sim.dx+sim.disp(i), sim.h(i));
   glEnd();
}

Gluvi::PanZoom2D cam(-0.1, -0.1, 1.5);

int main(int argc, char **argv)
{
   Gluvi::init("shallow1d", &argc, argv);

   glClearColor(0, 0, 0, 0);

   init_simulation();
   glutTimerFunc(50, advance_simulation, 0);

   MySlider f1("freq1", 30, sim.freq1),
            f2("freq2", 30, sim.freq2),
            a1("amp1", .25, sim.amp1),
            a2("amp2", .25, sim.amp2),
            s1("speed1", 5, sim.speed1),
            s2("speed2", 5, sim.speed2),
            h1("horiz1", 1, sim.horiz1),
            h2("horiz2", 1, sim.horiz2);
   Gluvi::root.list.push_back(&f1);
   Gluvi::root.list.push_back(&a1);
   Gluvi::root.list.push_back(&s1);
   Gluvi::root.list.push_back(&h1);
   Gluvi::root.list.push_back(&f2);
   Gluvi::root.list.push_back(&a2);
   Gluvi::root.list.push_back(&s2);
   Gluvi::root.list.push_back(&h2);

   Gluvi::camera=&cam;

   Gluvi::userDisplayFunc=display;

   Gluvi::run();
   return 0;
}
