#ifndef SIMULATION_H
#define SIMULATION_H

#include <cmath>

struct Simulation{
   int n;
   float dx;
   float t;
   float freq1, freq2;
   float amp1, amp2;
   float speed1, speed2;
   float horiz1, horiz2;
   float *hdata, *ddata;

   Simulation(void)
      : n(0), dx(0), t(0),
        freq1(0), freq2(0), amp1(0), amp2(0), speed1(0), speed2(0), horiz1(0), horiz2(0),
        hdata(0), ddata(0)
   {}

   ~Simulation(void)
   { delete[] hdata; delete[] ddata; }

   float &h(int i)
   { return hdata[i]; }

   float &disp(int i)
   { return ddata[i]; }

   void init(int n_)
   {
       delete[] hdata;
       delete[] ddata;
       n=n_;
       dx=1.0/n;
       t=0;
       hdata=new float[n];
       ddata=new float[n];
       advance(0);
   }

   void advance(float timestep)
   {
       t+=timestep;
       for(int i=0; i<n; ++i){
           float x=(i+.5)/n;
           h(i)=amp1*std::sin(freq1*(x-speed1*t))
               +amp2*std::sin(freq2*(x-speed2*t))
               +0.5f;
           disp(i)=horiz1*amp1*std::cos(freq1*(x-speed1*t))
                  +horiz2*amp2*std::cos(freq2*(x-speed2*t));
       }
   }
};

#endif
